@@ -413,7 +413,7 @@ func (b *MockExtractedRunKernelImageMixin) DisableTryKernel() error {
 // MockTrustedAssetsMixin implements the bootloader.TrustedAssetsBootloader
 // interface.
 type MockTrustedAssetsMixin struct {
-	TrustedAssetsList  []string
+	TrustedAssetsMap   map[string]string
 	TrustedAssetsErr   error
 	TrustedAssetsCalls int
 
@@ -491,20 +491,30 @@ func (b *MockTrustedAssetsMixin) CandidateCommandLine(pieces bootloader.CommandL
 	return glueCommandLine(pieces, b.CandidateStaticCommandLine)
 }
 
-func (b *MockTrustedAssetsMixin) TrustedAssets() ([]string, error) {
+func (b *MockTrustedAssetsMixin) DefaultCommandLine(candidate bool) (string, error) {
+	if b.CommandLineErr != nil {
+		return "", b.CommandLineErr
+	}
+	if candidate {
+		return b.CandidateStaticCommandLine, nil
+	}
+	return b.StaticCommandLine, nil
+}
+
+func (b *MockTrustedAssetsMixin) TrustedAssets() (map[string]string, error) {
 	b.TrustedAssetsCalls++
-	return b.TrustedAssetsList, b.TrustedAssetsErr
+	return b.TrustedAssetsMap, b.TrustedAssetsErr
 }
 
-func (b *MockTrustedAssetsMixin) RecoveryBootChain(kernelPath string) ([]bootloader.BootFile, error) {
+func (b *MockTrustedAssetsMixin) RecoveryBootChains(kernelPath string) ([][]bootloader.BootFile, error) {
 	b.RecoveryBootChainCalls = append(b.RecoveryBootChainCalls, kernelPath)
-	return b.RecoveryBootChainList, b.RecoveryBootChainErr
+	return [][]bootloader.BootFile{b.RecoveryBootChainList}, b.RecoveryBootChainErr
 }
 
-func (b *MockTrustedAssetsMixin) BootChain(runBl bootloader.Bootloader, kernelPath string) ([]bootloader.BootFile, error) {
+func (b *MockTrustedAssetsMixin) BootChains(runBl bootloader.Bootloader, kernelPath string) ([][]bootloader.BootFile, error) {
 	b.BootChainRunBl = append(b.BootChainRunBl, runBl)
 	b.BootChainKernelPath = append(b.BootChainKernelPath, kernelPath)
-	return b.BootChainList, b.BootChainErr
+	return [][]bootloader.BootFile{b.BootChainList}, b.BootChainErr
 }
 
 // MockRecoveryAwareTrustedAssetsBootloader implements the
