@@ -54,14 +54,14 @@ static void test_feature_enabled__missing_dir(void)
 	char subd[PATH_MAX];
 	sc_must_snprintf(subd, sizeof subd, "%s/absent", d);
 	sc_mock_feature_flag_dir(subd);
-	g_assert(!sc_feature_enabled(SC_FEATURE_PER_USER_MOUNT_NAMESPACE));
+	g_assert_false(sc_feature_enabled(SC_FEATURE_PER_USER_MOUNT_NAMESPACE));
 }
 
 static void test_feature_enabled__missing_file(void)
 {
 	const char *d = sc_testdir();
 	sc_mock_feature_flag_dir(d);
-	g_assert(!sc_feature_enabled(SC_FEATURE_PER_USER_MOUNT_NAMESPACE));
+	g_assert_false(sc_feature_enabled(SC_FEATURE_PER_USER_MOUNT_NAMESPACE));
 }
 
 static void test_feature_enabled__present_file(void)
@@ -70,9 +70,9 @@ static void test_feature_enabled__present_file(void)
 	sc_mock_feature_flag_dir(d);
 	char pname[PATH_MAX];
 	sc_must_snprintf(pname, sizeof pname, "%s/per-user-mount-namespace", d);
-	g_file_set_contents(pname, "", -1, NULL);
+	g_assert_true(g_file_set_contents(pname, "", -1, NULL));
 
-	g_assert(sc_feature_enabled(SC_FEATURE_PER_USER_MOUNT_NAMESPACE));
+	g_assert_true(sc_feature_enabled(SC_FEATURE_PER_USER_MOUNT_NAMESPACE));
 }
 
 static void test_feature_parallel_instances(void)
@@ -80,13 +80,13 @@ static void test_feature_parallel_instances(void)
 	const char *d = sc_testdir();
 	sc_mock_feature_flag_dir(d);
 
-	g_assert(!sc_feature_enabled(SC_FEATURE_PARALLEL_INSTANCES));
+	g_assert_false(sc_feature_enabled(SC_FEATURE_PARALLEL_INSTANCES));
 
 	char pname[PATH_MAX];
 	sc_must_snprintf(pname, sizeof pname, "%s/parallel-instances", d);
-	g_file_set_contents(pname, "", -1, NULL);
+	g_assert_true(g_file_set_contents(pname, "", -1, NULL));
 
-	g_assert(sc_feature_enabled(SC_FEATURE_PARALLEL_INSTANCES));
+	g_assert_true(sc_feature_enabled(SC_FEATURE_PARALLEL_INSTANCES));
 }
 
 static void test_feature_hidden_snap_folder(void)
@@ -94,13 +94,13 @@ static void test_feature_hidden_snap_folder(void)
 	const char *d = sc_testdir();
 	sc_mock_feature_flag_dir(d);
 
-	g_assert(!sc_feature_enabled(SC_FEATURE_HIDDEN_SNAP_FOLDER));
+	g_assert_false(sc_feature_enabled(SC_FEATURE_HIDDEN_SNAP_FOLDER));
 
 	char pname[PATH_MAX];
 	sc_must_snprintf(pname, sizeof pname, "%s/hidden-snap-folder", d);
-	g_file_set_contents(pname, "", -1, NULL);
+	g_assert_true(g_file_set_contents(pname, "", -1, NULL));
 
-	g_assert(sc_feature_enabled(SC_FEATURE_HIDDEN_SNAP_FOLDER));
+	g_assert_true(sc_feature_enabled(SC_FEATURE_HIDDEN_SNAP_FOLDER));
 }
 
 static void __attribute__((constructor)) init(void)
