@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2020 Canonical Ltd
+ * Copyright (C) 2014-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -35,12 +35,14 @@ import (
 	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/daemon"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/features"
 	"github.com/snapcore/snapd/interfaces/ifacetest"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/sandbox"
+	"github.com/snapcore/snapd/systemd"
 )
 
 var _ = check.Suite(&generalSuite{})
@@ -49,6 +51,10 @@ type generalSuite struct {
 	apiBaseSuite
 }
 
+func (s *generalSuite) expectChangesReadAccess() {
+	s.expectReadAccess(daemon.InterfaceOpenAccess{Interfaces: []string{"snap-refresh-observe"}})
+}
+
 func (s *generalSuite) TestRoot(c *check.C) {
 	s.daemon(c)
 
@@ -61,10 +67,10 @@ func (s *generalSuite) TestRoot(c *check.C) {
 	rec := httptest.NewRecorder()
 	s.req(c, req, nil).ServeHTTP(rec, nil)
 	c.Check(rec.Code, check.Equals, 200)
-	c.Check(rec.HeaderMap.Get("Content-Type"), check.Equals, "application/json")
+	c.Check(rec.Header().Get("Content-Type"), check.Equals, "application/json")
 
 	expected := []interface{}{"TBD"}
-	var rsp daemon.Resp
+	var rsp daemon.RespJSON
 	c.Assert(json.Unmarshal(rec.Body.Bytes(), &rsp), check.IsNil)
 	c.Check(rsp.Status, check.Equals, 200)
 	c.Check(rsp.Result, check.DeepEquals, expected)
@@ -86,6 +92,8 @@ func (s *generalSuite) TestSysInfo(c *check.C) {
 	tr := config.NewTransaction(st)
 	tr.Set("core", "refresh.schedule", "00:00-9:00/12:00-13:00")
 	tr.Set("core", "refresh.timer", "8:00~9:00/2")
+	tr.Set("core", "experimental.parallel-instances", "false")
+	tr.Set("core", "experimental.quota-groups", "true")
 	tr.Commit()
 	st.Unlock()
 
@@ -99,6 +107,9 @@ func (s *generalSuite) TestSysInfo(c *check.C) {
 	dirs.SetRootDir(dirs.GlobalRootDir)
 	restore = daemon.MockSystemdVirt("magic")
 	defer restore()
+	// Set systemd version <230 so QuotaGroups feature unsupported
+	restore = systemd.MockSystemdVersion(229, nil)
+	defer restore()
 
 	buildID := "this-is-my-build-id"
 	restore = daemon.MockBuildID(buildID)
@@ -107,7 +118,7 @@ func (s *generalSuite) TestSysInfo(c *check.C) {
 	rec := httptest.NewRecorder()
 	s.req(c, req, nil).ServeHTTP(rec, nil)
 	c.Check(rec.Code, check.Equals, 200)
-	c.Check(rec.HeaderMap.Get("Content-Type"), check.Equals, "application/json")
+	c.Check(rec.Header().Get("Content-Type"), check.Equals, "application/json")
 
 	expected := map[string]interface{}{
 		"series":  "16",
@@ -133,7 +144,7 @@ func (s *generalSuite) TestSysInfo(c *check.C) {
 		"virtualization":   "magic",
 		"system-mode":      "run",
 	}
-	var rsp daemon.Resp
+	var rsp daemon.RespJSON
 	c.Assert(json.Unmarshal(rec.Body.Bytes(), &rsp), check.IsNil)
 	c.Check(rsp.Status, check.Equals, 200)
 	c.Check(rsp.Type, check.Equals, daemon.ResponseTypeSync)
@@ -141,7 +152,48 @@ func (s *generalSuite) TestSysInfo(c *check.C) {
 	const kernelVersionKey = "kernel-version"
 	c.Check(rsp.Result.(map[string]interface{})[kernelVersionKey], check.Not(check.Equals), "")
 	delete(rsp.Result.(map[string]interface{}), kernelVersionKey)
+	// Extract "features" field and remove it from result; check it later.
+	const featuresKey = "features"
+	resultFeatures := rsp.Result.(map[string]interface{})[featuresKey]
+	c.Check(resultFeatures, check.Not(check.Equals), "")
+	delete(rsp.Result.(map[string]interface{}), featuresKey)
+
 	c.Check(rsp.Result, check.DeepEquals, expected)
+
+	// Check that "features" is map
+	featuresAll, ok := resultFeatures.(map[string]interface{})
+	c.Assert(ok, check.Equals, true)
+	// Ensure that Layouts exists and is feature.FeatureInfo
+	layoutsInfoRaw, exists := featuresAll[features.Layouts.String()]
+	c.Assert(exists, check.Equals, true)
+	layoutsInfo, ok := layoutsInfoRaw.(map[string]interface{})
+	c.Assert(ok, check.Equals, true, check.Commentf("%+v", layoutsInfoRaw))
+	// Ensure that Layouts is supported and enabled
+	c.Check(layoutsInfo["supported"], check.Equals, true)
+	_, exists = layoutsInfo["unsupported-reason"]
+	c.Check(exists, check.Equals, false)
+	c.Check(layoutsInfo["enabled"], check.Equals, true)
+	// Ensure that ParallelInstances exists and is a feature.FeatureInfo
+	parallelInstancesInfoRaw, exists := featuresAll[features.ParallelInstances.String()]
+	c.Assert(exists, check.Equals, true)
+	parallelInstancesInfo, ok := parallelInstancesInfoRaw.(map[string]interface{})
+	c.Assert(ok, check.Equals, true)
+	// Ensure that ParallelInstances is supported and not enabled
+	c.Check(parallelInstancesInfo["supported"], check.Equals, true)
+	_, exists = parallelInstancesInfo["unsupported-reason"]
+	c.Check(exists, check.Equals, false)
+	c.Check(parallelInstancesInfo["enabled"], check.Equals, false)
+	// Ensure that QuotaGroups exists and is a feature.FeatureInfo
+	quotaGroupsInfoRaw, exists := featuresAll[features.QuotaGroups.String()]
+	c.Assert(exists, check.Equals, true)
+	quotaGroupsInfo, ok := quotaGroupsInfoRaw.(map[string]interface{})
+	c.Assert(ok, check.Equals, true)
+	// Ensure that QuotaGroups is unsupported but enabled
+	c.Check(quotaGroupsInfo["supported"], check.Equals, false)
+	unsupportedReason, exists := quotaGroupsInfo["unsupported-reason"]
+	c.Check(exists, check.Equals, true)
+	c.Check(unsupportedReason, check.Not(check.Equals), "")
+	c.Check(quotaGroupsInfo["enabled"], check.Equals, true)
 }
 
 func (s *generalSuite) TestSysInfoLegacyRefresh(c *check.C) {
@@ -185,7 +237,7 @@ func (s *generalSuite) TestSysInfoLegacyRefresh(c *check.C) {
 	rec := httptest.NewRecorder()
 	s.req(c, req, nil).ServeHTTP(rec, nil)
 	c.Check(rec.Code, check.Equals, 200)
-	c.Check(rec.HeaderMap.Get("Content-Type"), check.Equals, "application/json")
+	c.Check(rec.Header().Get("Content-Type"), check.Equals, "application/json")
 
 	expected := map[string]interface{}{
 		"series":  "16",
@@ -214,12 +266,14 @@ func (s *generalSuite) TestSysInfoLegacyRefresh(c *check.C) {
 		"virtualization": "kvm",
 		"system-mode":    "run",
 	}
-	var rsp daemon.Resp
+	var rsp daemon.RespJSON
 	c.Assert(json.Unmarshal(rec.Body.Bytes(), &rsp), check.IsNil)
 	c.Check(rsp.Status, check.Equals, 200)
 	c.Check(rsp.Type, check.Equals, daemon.ResponseTypeSync)
 	const kernelVersionKey = "kernel-version"
 	delete(rsp.Result.(map[string]interface{}), kernelVersionKey)
+	const featuresKey = "features"
+	delete(rsp.Result.(map[string]interface{}), featuresKey)
 	c.Check(rsp.Result, check.DeepEquals, expected)
 }
 
@@ -265,7 +319,7 @@ func (s *generalSuite) testSysInfoSystemMode(c *check.C, mode string) {
 	rec := httptest.NewRecorder()
 	s.req(c, req, nil).ServeHTTP(rec, nil)
 	c.Check(rec.Code, check.Equals, 200)
-	c.Check(rec.HeaderMap.Get("Content-Type"), check.Equals, "application/json")
+	c.Check(rec.Header().Get("Content-Type"), check.Equals, "application/json")
 
 	expected := map[string]interface{}{
 		"series":  "16",
@@ -292,12 +346,14 @@ func (s *generalSuite) testSysInfoSystemMode(c *check.C, mode string) {
 		"architecture": arch.DpkgArchitecture(),
 		"system-mode":  mode,
 	}
-	var rsp daemon.Resp
+	var rsp daemon.RespJSON
 	c.Assert(json.Unmarshal(rec.Body.Bytes(), &rsp), check.IsNil)
 	c.Check(rsp.Status, check.Equals, 200)
 	c.Check(rsp.Type, check.Equals, daemon.ResponseTypeSync)
 	const kernelVersionKey = "kernel-version"
 	delete(rsp.Result.(map[string]interface{}), kernelVersionKey)
+	const featuresKey = "features"
+	delete(rsp.Result.(map[string]interface{}), featuresKey)
 	c.Check(rsp.Result, check.DeepEquals, expected)
 }
 
@@ -317,7 +373,12 @@ func (s *generalSuite) TestSysInfoIsManaged(c *check.C) {
 
 	st := d.Overlord().State()
 	st.Lock()
-	_, err := auth.NewUser(st, "someuser", "mymail@test.com", "macaroon", []string{"discharge"})
+	_, err := auth.NewUser(st, auth.NewUserParams{
+		Username:   "someuser",
+		Email:      "mymail@test.com",
+		Macaroon:   "macaroon",
+		Discharges: []string{"discharge"},
+	})
 	st.Unlock()
 	c.Assert(err, check.IsNil)
 
@@ -362,6 +423,7 @@ func (s *generalSuite) TestStateChangesDefaultToInProgress(c *check.C) {
 	defer restore()
 
 	// Setup
+	s.expectChangesReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -377,8 +439,10 @@ func (s *generalSuite) TestStateChangesDefaultToInProgress(c *check.C) {
 	c.Check(rsp.Status, check.Equals, 200)
 	c.Assert(rsp.Result, check.HasLen, 1)
 
-	res, err := rsp.MarshalJSON()
-	c.Assert(err, check.IsNil)
+	rec := httptest.NewRecorder()
+	rsp.ServeHTTP(rec, nil)
+	c.Assert(rec.Code, check.Equals, 200)
+	res := rec.Body.Bytes()
 
 	c.Check(string(res), check.Matches, `.*{"id":"\w+","kind":"install","summary":"install...","status":"Do","tasks":\[{"id":"\w+","kind":"download","summary":"1...","status":"Do","log":\["2016-04-21T01:02:03Z INFO l11","2016-04-21T01:02:03Z INFO l12"],"progress":{"label":"","done":0,"total":1},"spawn-time":"2016-04-21T01:02:03Z"}.*`)
 }
@@ -388,6 +452,7 @@ func (s *generalSuite) TestStateChangesInProgress(c *check.C) {
 	defer restore()
 
 	// Setup
+	s.expectChangesReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -403,8 +468,10 @@ func (s *generalSuite) TestStateChangesInProgress(c *check.C) {
 	c.Check(rsp.Status, check.Equals, 200)
 	c.Assert(rsp.Result, check.HasLen, 1)
 
-	res, err := rsp.MarshalJSON()
-	c.Assert(err, check.IsNil)
+	rec := httptest.NewRecorder()
+	rsp.ServeHTTP(rec, nil)
+	c.Assert(rec.Code, check.Equals, 200)
+	res := rec.Body.Bytes()
 
 	c.Check(string(res), check.Matches, `.*{"id":"\w+","kind":"install","summary":"install...","status":"Do","tasks":\[{"id":"\w+","kind":"download","summary":"1...","status":"Do","log":\["2016-04-21T01:02:03Z INFO l11","2016-04-21T01:02:03Z INFO l12"],"progress":{"label":"","done":0,"total":1},"spawn-time":"2016-04-21T01:02:03Z"}.*],"ready":false,"spawn-time":"2016-04-21T01:02:03Z"}.*`)
 }
@@ -414,6 +481,7 @@ func (s *generalSuite) TestStateChangesAll(c *check.C) {
 	defer restore()
 
 	// Setup
+	s.expectChangesReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -429,8 +497,10 @@ func (s *generalSuite) TestStateChangesAll(c *check.C) {
 	c.Check(rsp.Status, check.Equals, 200)
 	c.Assert(rsp.Result, check.HasLen, 2)
 
-	res, err := rsp.MarshalJSON()
-	c.Assert(err, check.IsNil)
+	rec := httptest.NewRecorder()
+	rsp.ServeHTTP(rec, nil)
+	c.Assert(rec.Code, check.Equals, 200)
+	res := rec.Body.Bytes()
 
 	c.Check(string(res), check.Matches, `.*{"id":"\w+","kind":"install","summary":"install...","status":"Do","tasks":\[{"id":"\w+","kind":"download","summary":"1...","status":"Do","log":\["2016-04-21T01:02:03Z INFO l11","2016-04-21T01:02:03Z INFO l12"],"progress":{"label":"","done":0,"total":1},"spawn-time":"2016-04-21T01:02:03Z"}.*],"ready":false,"spawn-time":"2016-04-21T01:02:03Z"}.*`)
 	c.Check(string(res), check.Matches, `.*{"id":"\w+","kind":"remove","summary":"remove..","status":"Error","tasks":\[{"id":"\w+","kind":"unlink","summary":"1...","status":"Error","log":\["2016-04-21T01:02:03Z ERROR rm failed"],"progress":{"label":"","done":1,"total":1},"spawn-time":"2016-04-21T01:02:03Z","ready-time":"2016-04-21T01:02:03Z"}.*],"ready":true,"err":"[^"]+".*`)
@@ -441,6 +511,7 @@ func (s *generalSuite) TestStateChangesReady(c *check.C) {
 	defer restore()
 
 	// Setup
+	s.expectChangesReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -456,8 +527,10 @@ func (s *generalSuite) TestStateChangesReady(c *check.C) {
 	c.Check(rsp.Status, check.Equals, 200)
 	c.Assert(rsp.Result, check.HasLen, 1)
 
-	res, err := rsp.MarshalJSON()
-	c.Assert(err, check.IsNil)
+	rec := httptest.NewRecorder()
+	rsp.ServeHTTP(rec, nil)
+	c.Assert(rec.Code, check.Equals, 200)
+	res := rec.Body.Bytes()
 
 	c.Check(string(res), check.Matches, `.*{"id":"\w+","kind":"remove","summary":"remove..","status":"Error","tasks":\[{"id":"\w+","kind":"unlink","summary":"1...","status":"Error","log":\["2016-04-21T01:02:03Z ERROR rm failed"],"progress":{"label":"","done":1,"total":1},"spawn-time":"2016-04-21T01:02:03Z","ready-time":"2016-04-21T01:02:03Z"}.*],"ready":true,"err":"[^"]+".*`)
 }
@@ -467,6 +540,7 @@ func (s *generalSuite) TestStateChangesForSnapName(c *check.C) {
 	defer restore()
 
 	// Setup
+	s.expectChangesReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -486,8 +560,9 @@ func (s *generalSuite) TestStateChangesForSnapName(c *check.C) {
 	c.Assert(res, check.HasLen, 1)
 	c.Check(res[0].Kind, check.Equals, `install`)
 
-	_, err = rsp.MarshalJSON()
-	c.Assert(err, check.IsNil)
+	rec := httptest.NewRecorder()
+	rsp.ServeHTTP(rec, nil)
+	c.Assert(rec.Code, check.Equals, 200)
 }
 
 func (s *generalSuite) TestStateChangesForSnapNameWithApp(c *check.C) {
@@ -495,6 +570,7 @@ func (s *generalSuite) TestStateChangesForSnapNameWithApp(c *check.C) {
 	defer restore()
 
 	// Setup
+	s.expectChangesReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -520,8 +596,9 @@ func (s *generalSuite) TestStateChangesForSnapNameWithApp(c *check.C) {
 	c.Assert(res, check.HasLen, 1)
 	c.Check(res[0].Kind, check.Equals, `service-control`)
 
-	_, err = rsp.MarshalJSON()
-	c.Assert(err, check.IsNil)
+	rec := httptest.NewRecorder()
+	rsp.ServeHTTP(rec, nil)
+	c.Assert(rec.Code, check.Equals, 200)
 }
 
 func (s *generalSuite) TestStateChange(c *check.C) {
@@ -529,6 +606,7 @@ func (s *generalSuite) TestStateChange(c *check.C) {
 	defer restore()
 
 	// Setup
+	s.expectChangesReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -599,6 +677,7 @@ func (s *generalSuite) TestStateChangeAbort(c *check.C) {
 	defer restore()
 
 	// Setup
+	s.expectChangesReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -664,6 +743,7 @@ func (s *generalSuite) TestStateChangeAbortIsReady(c *check.C) {
 	defer restore()
 
 	// Setup
+	s.expectChangesReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -678,14 +758,13 @@ func (s *generalSuite) TestStateChangeAbortIsReady(c *check.C) {
 	// Execute
 	req, err := http.NewRequest("POST", "/v2/changes/"+ids[0], buf)
 	c.Assert(err, check.IsNil)
-	rsp := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil)
 	rec := httptest.NewRecorder()
-	rsp.ServeHTTP(rec, req)
+	rspe.ServeHTTP(rec, req)
 
 	// Verify
 	c.Check(rec.Code, check.Equals, 400)
-	c.Check(rsp.Status, check.Equals, 400)
-	c.Check(rsp.Result, check.NotNil)
+	c.Check(rspe.Status, check.Equals, 400)
 
 	var body map[string]interface{}
 	err = json.Unmarshal(rec.Body.Bytes(), &body)
