@@ -24,8 +24,11 @@ import (
 
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
+	"github.com/snapcore/snapd/testutil"
+	"github.com/snapcore/snapd/timings"
 )
 
 type BackendSuite struct {
@@ -34,18 +37,41 @@ type BackendSuite struct {
 	Iface           *TestInterface
 	RootDir         string
 	restoreSanitize func()
+
+	meas *timings.Span
+	testutil.BaseTest
 }
 
+// CoreSnapInfo is set in SetupSuite
+var DefaultInitializeOpts = &interfaces.SecurityBackendOptions{}
+
 func (s *BackendSuite) SetUpTest(c *C) {
+	coreSnapPlaceInfo := snap.MinimalPlaceInfo("core", snap.Revision{N: 123})
+	snInfo, ok := coreSnapPlaceInfo.(*snap.Info)
+	c.Assert(ok, Equals, true)
+	DefaultInitializeOpts.CoreSnapInfo = snInfo
+
+	snapdSnapPlaceInfo := snap.MinimalPlaceInfo("snapd", snap.Revision{N: 321})
+	snInfo, ok = snapdSnapPlaceInfo.(*snap.Info)
+	c.Assert(ok, Equals, true)
+	DefaultInitializeOpts.SnapdSnapInfo = snInfo
+
 	// Isolate this test to a temporary directory
 	s.RootDir = c.MkDir()
 	dirs.SetRootDir(s.RootDir)
+
+	restore := osutil.MockMountInfo("")
+	s.AddCleanup(restore)
+
 	// Create a fresh repository for each test
 	s.Repo = interfaces.NewRepository()
 	s.Iface = &TestInterface{InterfaceName: "iface"}
 	err := s.Repo.AddInterface(s.Iface)
 	c.Assert(err, IsNil)
 
+	perf := timings.New(nil)
+	s.meas = perf.StartSpan("", "")
+
 	s.restoreSanitize = snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {})
 }
 
@@ -65,6 +91,17 @@ slots:
     slot:
         interface: iface
 `
+const SambaYamlV1Core20Base = `
+name: samba
+base: core20
+version: 1
+developer: acme
+apps:
+    smbd:
+slots:
+    slot:
+        interface: iface
+`
 const SambaYamlV1WithNmbd = `
 name: samba
 version: 1
@@ -144,6 +181,14 @@ slots:
         interface: iface
 `
 
+const SomeSnapYamlV1 = `
+name: some-snap
+version: 1
+developer: acme
+apps:
+    someapp:
+`
+
 // Support code for tests
 
 // InstallSnap "installs" a snap from YAML.
@@ -152,6 +197,8 @@ func (s *BackendSuite) InstallSnap(c *C, opts interfaces.ConfinementOptions, ins
 		Revision: snap.R(revision),
 	})
 
+	appSet := interfaces.NewSnapAppSet(snapInfo)
+
 	if instanceName != "" {
 		_, instanceKey := snap.SplitInstanceName(instanceName)
 		snapInfo.InstanceKey = instanceKey
@@ -159,22 +206,29 @@ func (s *BackendSuite) InstallSnap(c *C, opts interfaces.ConfinementOptions, ins
 	}
 
 	s.addPlugsSlots(c, snapInfo)
-	err := s.Backend.Setup(snapInfo, opts, s.Repo)
+	err := s.Backend.Setup(appSet, opts, s.Repo, s.meas)
 	c.Assert(err, IsNil)
 	return snapInfo
 }
 
 // UpdateSnap "updates" an existing snap from YAML.
 func (s *BackendSuite) UpdateSnap(c *C, oldSnapInfo *snap.Info, opts interfaces.ConfinementOptions, snapYaml string, revision int) *snap.Info {
+	newSnapInfo, err := s.UpdateSnapMaybeErr(c, oldSnapInfo, opts, snapYaml, revision)
+	c.Assert(err, IsNil)
+	return newSnapInfo
+}
+
+// UpdateSnapMaybeErr "updates" an existing snap from YAML, this might error.
+func (s *BackendSuite) UpdateSnapMaybeErr(c *C, oldSnapInfo *snap.Info, opts interfaces.ConfinementOptions, snapYaml string, revision int) (*snap.Info, error) {
 	newSnapInfo := snaptest.MockInfo(c, snapYaml, &snap.SideInfo{
 		Revision: snap.R(revision),
 	})
+	appSet := interfaces.NewSnapAppSet(newSnapInfo)
 	c.Assert(newSnapInfo.InstanceName(), Equals, oldSnapInfo.InstanceName())
 	s.removePlugsSlots(c, oldSnapInfo)
 	s.addPlugsSlots(c, newSnapInfo)
-	err := s.Backend.Setup(newSnapInfo, opts, s.Repo)
-	c.Assert(err, IsNil)
-	return newSnapInfo
+	err := s.Backend.Setup(appSet, opts, s.Repo, s.meas)
+	return newSnapInfo, err
 }
 
 // RemoveSnap "removes" an "installed" snap.
