@@ -645,7 +645,7 @@ func (r *Repository) Disconnect(plugSnapName, plugName, slotSnapName, slotName s
 	r.m.Lock()
 	defer r.m.Unlock()
 
-	// Sanity check
+	// Validity check
 	if plugSnapName == "" {
 		return fmt.Errorf("cannot disconnect, plug snap name is empty")
 	}
@@ -912,7 +912,7 @@ func (r *Repository) Interfaces() *Interfaces {
 }
 
 // SnapSpecification returns the specification of a given snap in a given security system.
-func (r *Repository) SnapSpecification(securitySystem SecuritySystem, snapName string) (Specification, error) {
+func (r *Repository) SnapSpecification(securitySystem SecuritySystem, appSet *SnapAppSet) (Specification, error) {
 	r.m.Lock()
 	defer r.m.Unlock()
 
@@ -923,11 +923,13 @@ func (r *Repository) SnapSpecification(securitySystem SecuritySystem, snapName s
 			break
 		}
 	}
+
+	snapName := appSet.InstanceName()
 	if backend == nil {
 		return nil, fmt.Errorf("cannot handle interfaces of snap %q, security system %q is not known", snapName, securitySystem)
 	}
 
-	spec := backend.NewSpecification()
+	spec := backend.NewSpecification(appSet)
 
 	// XXX: If either of the AddConnected{Plug,Slot} methods for a connection
 	// fail resiliently as-in they can never succeed (such as the case where a
