@@ -19,25 +19,10 @@
 
 package disks
 
-import "fmt"
+var (
+	UnregisterDeviceMapperBackResolver = unregisterDeviceMapperBackResolver
 
-func MockUdevPropertiesForDevice(new func(string) (map[string]string, error)) (restore func()) {
-	old := udevadmProperties
-	// for better testing we mock the udevadm command output so that we still
-	// test the parsing
-	udevadmProperties = func(dev string) ([]byte, error) {
-		props, err := new(dev)
-		if err != nil {
-			return []byte(err.Error()), err
-		}
-		// put it into udevadm format output, i.e. "KEY=VALUE\n"
-		output := ""
-		for k, v := range props {
-			output += fmt.Sprintf("%s=%s\n", k, v)
-		}
-		return []byte(output), nil
-	}
-	return func() {
-		udevadmProperties = old
-	}
-}
+	CryptLuks2DeviceMapperBackResolver = cryptLuks2DeviceMapperBackResolver
+
+	FilesystemTypeForPartition = filesystemTypeForPartition
+)
