@@ -21,6 +21,7 @@ package hookstate
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -33,10 +34,8 @@ import (
 	"gopkg.in/tomb.v2"
 
 	"github.com/snapcore/snapd/dirs"
-	"github.com/snapcore/snapd/errtracker"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
-	"github.com/snapcore/snapd/overlord/configstate/settings"
 	"github.com/snapcore/snapd/overlord/restart"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
@@ -88,14 +87,13 @@ type HookSetup struct {
 	Optional    bool          `json:"optional,omitempty"`     // do not error if script is missing
 	Always      bool          `json:"always,omitempty"`       // run handler even if script is missing
 	IgnoreError bool          `json:"ignore-error,omitempty"` // do not run handler's Error() on error
-	TrackError  bool          `json:"track-error,omitempty"`  // report hook error to oopsie
 }
 
 // Manager returns a new HookManager.
 func Manager(s *state.State, runner *state.TaskRunner) (*HookManager, error) {
 	// Make sure we only run 1 hook task for given snap at a time
 	runner.AddBlocked(func(thisTask *state.Task, running []*state.Task) bool {
-		// check if we're a hook task, probably not needed but let's take extra care
+		// check if we're a hook task
 		if thisTask.Kind() != "run-hook" {
 			return false
 		}
@@ -135,7 +133,7 @@ func Manager(s *state.State, runner *state.TaskRunner) (*HookManager, error) {
 
 	setupHooks(manager)
 
-	snapstate.AddAffectedSnapsByAttr("hook-setup", manager.hookAffectedSnaps)
+	snapstate.RegisterAffectedSnapsByAttr("hook-setup", manager.hookAffectedSnaps)
 
 	return manager, nil
 }
@@ -226,7 +224,7 @@ func hookSetup(task *state.Task, key string) (*HookSetup, *snapstate.SnapState,
 
 	var snapst snapstate.SnapState
 	err = snapstate.Get(task.State(), hooksup.Snap, &snapst)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, nil, fmt.Errorf("cannot handle %q snap: %v", hooksup.Snap, err)
 	}
 
@@ -276,7 +274,7 @@ func (m *HookManager) undoRunHook(task *state.Task, tomb *tomb.Tomb) error {
 	hooksup, snapst, err := hookSetup(task, "undo-hook-setup")
 	task.State().Unlock()
 	if err != nil {
-		if err == state.ErrNoState {
+		if errors.Is(err, state.ErrNoState) {
 			// no undo hook setup
 			return nil
 		}
@@ -403,9 +401,7 @@ func (m *HookManager) runHook(context *Context, snapst *snapstate.SnapState, hoo
 		output, err = runHook(context, tomb)
 	}
 	if err != nil {
-		if hooksup.TrackError {
-			trackHookError(context, output, err)
-		}
+		// TODO: telemetry about errors here
 		err = osutil.OutputErr(output, err)
 		if hooksup.IgnoreError {
 			context.Lock()
@@ -495,28 +491,3 @@ func runHookAndWait(snapName string, revision snap.Revision, hookName, hookConte
 
 	return osutil.RunAndWait(argv, env, timeout, tomb)
 }
-
-var errtrackerReport = errtracker.Report
-
-func trackHookError(context *Context, output []byte, err error) {
-	errmsg := fmt.Sprintf("hook %s in snap %q failed: %v", context.HookName(), context.InstanceName(), osutil.OutputErr(output, err))
-	dupSig := fmt.Sprintf("hook:%s:%s:%s\n%s", context.InstanceName(), context.HookName(), err, output)
-	extra := map[string]string{
-		"HookName": context.HookName(),
-	}
-	if context.setup.IgnoreError {
-		extra["IgnoreError"] = "1"
-	}
-
-	context.state.Lock()
-	problemReportsDisabled := settings.ProblemReportsDisabled(context.state)
-	context.state.Unlock()
-	if !problemReportsDisabled {
-		oopsid, err := errtrackerReport(context.InstanceName(), errmsg, dupSig, extra)
-		if err == nil {
-			logger.Noticef("Reported hook failure from %q for snap %q as %s", context.HookName(), context.InstanceName(), oopsid)
-		} else {
-			logger.Debugf("Cannot report hook failure: %s", err)
-		}
-	}
-}
