@@ -40,6 +40,7 @@ import (
 	"github.com/snapcore/snapd/overlord/ifacestate"
 	"github.com/snapcore/snapd/overlord/ifacestate/udevmonitor"
 	"github.com/snapcore/snapd/overlord/snapstate"
+	"github.com/snapcore/snapd/overlord/snapstate/snapstatetest"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
@@ -167,7 +168,7 @@ func (s *hotplugSuite) SetUpTest(c *C) {
 	s.state.Lock()
 	snapstate.Set(s.state, "core", &snapstate.SnapState{
 		Active:   true,
-		Sequence: []*snap.SideInfo{si},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{si}),
 		Current:  snap.R(1),
 		SnapType: "os",
 	})
@@ -354,7 +355,7 @@ slots:
 `, gadgetSideInfo)
 	snapstate.Set(s.state, "the-gadget", &snapstate.SnapState{
 		Active:   true,
-		Sequence: []*snap.SideInfo{&gadgetInfo.SideInfo},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{&gadgetInfo.SideInfo}),
 		Current:  snap.R(1),
 		SnapType: "gadget"})
 	st.Unlock()
@@ -444,7 +445,7 @@ func (s *hotplugSuite) TestHotplugAddWithAutoconnect(c *C) {
 	c.Assert(repo.AddPlug(testSnap.Plugs["plug"]), IsNil)
 	snapstate.Set(s.state, "consumer", &snapstate.SnapState{
 		Active:   true,
-		Sequence: []*snap.SideInfo{si},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{si}),
 		Current:  snap.R(1),
 		SnapType: "app",
 	})
@@ -518,7 +519,7 @@ func (s *hotplugSuite) TestHotplugRemove(c *C) {
 	}), IsNil)
 	snapstate.Set(s.state, "consumer", &snapstate.SnapState{
 		Active:   true,
-		Sequence: []*snap.SideInfo{si},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{si}),
 		Current:  snap.R(1),
 		SnapType: "app",
 	})
@@ -603,7 +604,7 @@ func (s *hotplugSuite) TestHotplugEnumerationDone(c *C) {
 	}), IsNil)
 	snapstate.Set(s.state, "consumer", &snapstate.SnapState{
 		Active:   true,
-		Sequence: []*snap.SideInfo{si},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{si}),
 		Current:  snap.R(1),
 		SnapType: "app"})
 
@@ -634,7 +635,7 @@ func (s *hotplugSuite) TestHotplugEnumerationDone(c *C) {
 			"interface":   "test-a",
 			"hotplug-key": "yet-another-device"}})
 
-	// sanity
+	// precondition
 	slot, _ := repo.SlotForHotplugKey("test-a", "key-other-device")
 	c.Assert(slot, NotNil)
 
@@ -705,7 +706,7 @@ func (s *hotplugSuite) TestHotplugDeviceUpdate(c *C) {
 	}), IsNil)
 	snapstate.Set(s.state, "consumer", &snapstate.SnapState{
 		Active:   true,
-		Sequence: []*snap.SideInfo{si},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{si}),
 		Current:  snap.R(1),
 		SnapType: "app"})
 
@@ -807,7 +808,7 @@ func (s *hotplugSuite) TestDefaultDeviceKey(c *C) {
 	key, err := ifacestate.DefaultDeviceKey(di, 0)
 	c.Assert(err, IsNil)
 
-	// sanity check
+	// validity check
 	c.Check(key, HasLen, 65)
 	c.Check(key, Equals, snap.HotplugKey("08bcbdcda3fee3534c0288506d9b75d4e26fe3692a36a11e75d05eac9ebf5ca7d"))
 	c.Assert(key, Equals, keyHelper("ID_V4L_PRODUCT\x00v4lproduct\x00ID_VENDOR_ID\x00vendor\x00ID_MODEL_ID\x00model\x00ID_SERIAL\x00serial\x00"))
