@@ -31,8 +31,10 @@ import (
 	"github.com/snapcore/snapd/asserts/snapasserts"
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/ifacestate/ifacerepo"
 	"github.com/snapcore/snapd/overlord/snapstate"
+	"github.com/snapcore/snapd/overlord/snapstate/sequence"
 	"github.com/snapcore/snapd/overlord/snapstate/snapstatetest"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/release"
@@ -70,15 +72,19 @@ func (s *prereqSuite) SetUpTest(c *C) {
 	restoreCheckFreeSpace := snapstate.MockOsutilCheckFreeSpace(func(string, uint64) error { return nil })
 	s.AddCleanup(restoreCheckFreeSpace)
 
-	restoreInstallSize := snapstate.MockInstallSize(func(st *state.State, snaps []snapstate.MinimalInstallInfo, userID int) (uint64, error) {
+	restoreInstallSize := snapstate.MockInstallSize(func(st *state.State, snaps []snapstate.MinimalInstallInfo, userID int, prqt snapstate.PrereqTracker) (uint64, error) {
 		return 0, nil
 	})
 	s.AddCleanup(restoreInstallSize)
 
-	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVs *asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
+	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 		return nil, nil
 	})
 	s.AddCleanup(restore)
+
+	s.AddCleanup(osutil.MockMountInfo(``))
+
+	s.AddCleanup(snapstate.MockEnsuredMountsUpdated(s.snapmgr, true))
 }
 
 func (s *prereqSuite) TestDoPrereqNothingToDo(c *C) {
@@ -89,7 +95,7 @@ func (s *prereqSuite) TestDoPrereqNothingToDo(c *C) {
 		Revision: snap.R(1),
 	}
 	snapstate.Set(s.state, "core", &snapstate.SnapState{
-		Sequence: []*snap.SideInfo{si1},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{si1}),
 		Current:  si1.Revision,
 	})
 
@@ -100,7 +106,7 @@ func (s *prereqSuite) TestDoPrereqNothingToDo(c *C) {
 			Revision: snap.R(33),
 		},
 	})
-	s.state.NewChange("dummy", "...").AddTask(t)
+	s.state.NewChange("sample", "...").AddTask(t)
 	s.state.Unlock()
 
 	s.se.Ensure()
@@ -124,7 +130,7 @@ func (s *prereqSuite) TestDoPrereqWithBaseNone(c *C) {
 		Base:               "none",
 		PrereqContentAttrs: map[string][]string{"prereq1": {"some-content"}},
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 	s.state.Unlock()
 
@@ -170,7 +176,7 @@ func (s *prereqSuite) TestDoPrereqManyTransactional(c *C) {
 	// Set lane to make sure new tasks will match this one
 	lane := s.state.NewLane()
 	t.JoinLane(lane)
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 	s.state.Unlock()
 
@@ -215,7 +221,7 @@ func (s *prereqSuite) TestDoPrereqTransactionalFailTooManyLanes(c *C) {
 	// There should be only one lane in a transactional change
 	t.JoinLane(s.state.NewLane())
 	t.JoinLane(s.state.NewLane())
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 	s.state.Unlock()
 
@@ -232,9 +238,9 @@ func (s *prereqSuite) TestDoPrereqTalksToStoreAndQueues(c *C) {
 
 	snapstate.Set(s.state, "core", &snapstate.SnapState{
 		Active: true,
-		Sequence: []*snap.SideInfo{
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{
 			{RealName: "core", Revision: snap.R(1)},
-		},
+		}),
 		Current:  snap.R(1),
 		SnapType: "os",
 	})
@@ -249,7 +255,7 @@ func (s *prereqSuite) TestDoPrereqTalksToStoreAndQueues(c *C) {
 		Base:               "some-base",
 		PrereqContentAttrs: map[string][]string{"prereq1": {"some-content"}, "prereq2": {"other-content"}},
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 	s.state.Unlock()
 
@@ -337,7 +343,7 @@ func (s *prereqSuite) TestDoPrereqRetryWhenBaseInFlight(c *C) {
 			var snapst snapstate.SnapState
 			snapstate.Get(st, snapsup.InstanceName(), &snapst)
 			snapst.Current = snapsup.Revision()
-			snapst.Sequence = append(snapst.Sequence, snapsup.SideInfo)
+			snapst.Sequence.Revisions = append(snapst.Sequence.Revisions, sequence.NewRevisionSideState(snapsup.SideInfo, nil))
 			snapstate.Set(st, snapsup.InstanceName(), &snapst)
 
 			// check that prerequisites task is not done yet, it must wait for core.
@@ -364,7 +370,7 @@ func (s *prereqSuite) TestDoPrereqRetryWhenBaseInFlight(c *C) {
 		},
 	})
 
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(prereqTask)
 	chg.AddTask(tCore)
 
@@ -386,13 +392,13 @@ func (s *prereqSuite) TestDoPrereqRetryWhenBaseInFlight(c *C) {
 		}
 	}
 
-	// sanity check, exactly two calls to link-snap due to retry error on 1st call
+	// validity check, exactly two calls to link-snap due to retry error on 1st call
 	c.Check(calls, Equals, 2)
 
 	c.Check(tCore.Status(), Equals, state.DoneStatus)
 	c.Check(prereqTask.Status(), Equals, state.DoneStatus)
 
-	// sanity
+	// validity
 	c.Check(chg.Status(), Equals, state.DoneStatus)
 }
 
@@ -405,9 +411,9 @@ func (s *prereqSuite) TestDoPrereqChannelEnvvars(c *C) {
 
 	snapstate.Set(s.state, "core", &snapstate.SnapState{
 		Active: true,
-		Sequence: []*snap.SideInfo{
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{
 			{RealName: "core", Revision: snap.R(1)},
-		},
+		}),
 		Current:  snap.R(1),
 		SnapType: "os",
 	})
@@ -422,7 +428,7 @@ func (s *prereqSuite) TestDoPrereqChannelEnvvars(c *C) {
 		Base:               "some-base",
 		PrereqContentAttrs: map[string][]string{"prereq1": {"some-content"}, "prereq2": {"other-content"}},
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 	s.state.Unlock()
 
@@ -489,7 +495,7 @@ func (s *prereqSuite) TestDoPrereqNothingToDoForBase(c *C) {
 			},
 			Type: typ,
 		})
-		s.state.NewChange("dummy", "...").AddTask(t)
+		s.state.NewChange("sample", "...").AddTask(t)
 		s.state.Unlock()
 
 		s.se.Ensure()
@@ -513,7 +519,7 @@ func (s *prereqSuite) TestDoPrereqNothingToDoForSnapdSnap(c *C) {
 			Revision: snap.R(1),
 		},
 	})
-	s.state.NewChange("dummy", "...").AddTask(t)
+	s.state.NewChange("sample", "...").AddTask(t)
 	s.state.Unlock()
 
 	s.se.Ensure()
@@ -533,7 +539,7 @@ func (s *prereqSuite) TestDoPrereqCore16wCoreNothingToDo(c *C) {
 		Revision: snap.R(1),
 	}
 	snapstate.Set(s.state, "core", &snapstate.SnapState{
-		Sequence: []*snap.SideInfo{si1},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{si1}),
 		Current:  si1.Revision,
 	})
 
@@ -545,7 +551,7 @@ func (s *prereqSuite) TestDoPrereqCore16wCoreNothingToDo(c *C) {
 		},
 		Base: "core16",
 	})
-	s.state.NewChange("dummy", "...").AddTask(t)
+	s.state.NewChange("sample", "...").AddTask(t)
 	s.state.Unlock()
 
 	s.se.Ensure()
@@ -571,7 +577,7 @@ func (s *prereqSuite) testDoPrereqNoCorePullsInSnaps(c *C, base string) {
 		},
 		Base: base,
 	})
-	s.state.NewChange("dummy", "...").AddTask(t)
+	s.state.NewChange("sample", "...").AddTask(t)
 	s.state.Unlock()
 
 	s.se.Ensure()
@@ -635,7 +641,7 @@ func (s *prereqSuite) TestDoPrereqBaseIsNotBase(c *C) {
 		Base:               "app-snap",
 		PrereqContentAttrs: map[string][]string{"prereq1": {"some-content"}},
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 	s.state.Unlock()
 
@@ -664,7 +670,7 @@ func (s *prereqSuite) TestDoPrereqBaseNoRevision(c *C) {
 		Base:               "some-base",
 		PrereqContentAttrs: map[string][]string{"prereq1": {"some-content"}},
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 	s.state.Unlock()
 
@@ -692,7 +698,7 @@ func (s *prereqSuite) TestDoPrereqNoRevision(c *C) {
 		Channel:            "beta",
 		PrereqContentAttrs: map[string][]string{"prereq1": {"some-content"}},
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 	s.state.Unlock()
 
@@ -720,7 +726,7 @@ func (s *prereqSuite) TestDoPrereqSnapdNoRevision(c *C) {
 		Base:    "core18",
 		Channel: "beta",
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 	s.state.Unlock()
 
@@ -732,3 +738,170 @@ func (s *prereqSuite) TestDoPrereqSnapdNoRevision(c *C) {
 	c.Check(chg.Status(), Equals, state.ErrorStatus)
 	c.Check(chg.Err(), ErrorMatches, `cannot perform the following tasks:\n.*- test \(cannot install system snap "snapd": no snap revision available as specified\)`)
 }
+
+func (s *prereqSuite) TestPreReqContentAttrsNotSatisfied(c *C) {
+	snapstate.AutoAliases = func(*state.State, *snap.Info) (map[string]string, error) {
+		return nil, nil
+	}
+	s.AddCleanup(func() { snapstate.AutoAliases = nil })
+
+	st := s.state
+	st.Lock()
+
+	// mock the snap which is the default-provider
+	mockInstalledSnap(c, st, `name: some-snap`, false)
+
+	t := s.state.NewTask("prerequisites", "test")
+	t.Set("snap-setup", &snapstate.SnapSetup{
+		SideInfo: &snap.SideInfo{
+			RealName: "foo",
+			Revision: snap.R(33),
+		},
+		Channel:            "beta",
+		Base:               "core",
+		PrereqContentAttrs: map[string][]string{"some-snap": {"this-does-not-match"}},
+	})
+	chg := s.state.NewChange("sample", "...")
+	chg.AddTask(t)
+	st.Unlock()
+
+	s.se.Ensure()
+	s.se.Wait()
+
+	st.Lock()
+	defer st.Unlock()
+
+	// As we are not seeding, expect no messages being logged for the task. Instead
+	// the update should have added a lot of tasks, so the resulting change should
+	// contain the update for some-snap.
+	c.Check(chg.Err(), IsNil)
+	c.Check(len(chg.Tasks()) > 1, Equals, true)
+
+	// Expect the initial prerequisites task to have completed
+	c.Check(chg.Tasks()[0].Kind(), Equals, "prerequisites")
+	c.Check(chg.Tasks()[0].Status(), Equals, state.DoneStatus)
+	c.Check(chg.Tasks()[0].Log(), HasLen, 0)
+}
+
+func (s *prereqSuite) TestPreReqContentAttrsNotSatisfiedSeeding(c *C) {
+	snapstate.AutoAliases = func(*state.State, *snap.Info) (map[string]string, error) {
+		return nil, nil
+	}
+	s.AddCleanup(func() { snapstate.AutoAliases = nil })
+
+	// mock that we are not seeded
+	r := snapstatetest.MockDeviceModelAndMode(DefaultModel(), "install")
+	defer r()
+
+	st := s.state
+	st.Lock()
+	st.Set("seeded", false)
+
+	// mock the snap which is the default-provider
+	mockInstalledSnap(c, st, `name: some-snap`, false)
+
+	t := s.state.NewTask("prerequisites", "test")
+	t.Set("snap-setup", &snapstate.SnapSetup{
+		SideInfo: &snap.SideInfo{
+			RealName: "foo",
+			Revision: snap.R(33),
+		},
+		Channel:            "beta",
+		Base:               "core",
+		PrereqContentAttrs: map[string][]string{"some-snap": {"this-does-not-match"}},
+	})
+	chg := s.state.NewChange("sample", "...")
+	chg.AddTask(t)
+	st.Unlock()
+
+	s.se.Ensure()
+	s.se.Wait()
+
+	st.Lock()
+	defer st.Unlock()
+
+	// We expect the change to complete without error, even if updating a snap is not
+	// allowed during seeding. Instead what we expect, is a task log message noting that
+	// updating the snap was not allowed due to the seeding stage.
+	c.Check(chg.Err(), IsNil)
+	c.Assert(chg.Tasks(), HasLen, 1)
+	c.Assert(chg.Tasks()[0].Log(), HasLen, 1)
+	c.Check(chg.Tasks()[0].Log()[0], testutil.Contains, `cannot update "some-snap" during seeding, will not have required content "this-does-not-match": too early for operation, device not yet seeded or device model not acknowledged`)
+}
+
+func (s *prereqSuite) TestDoPrereqSkipDuringRemodel(c *C) {
+	s.state.Lock()
+
+	restore := snapstatetest.MockDeviceContext(&snapstatetest.TrivialDeviceContext{
+		Remodeling: true,
+	})
+	defer restore()
+
+	// install snapd so that prerequisites handler won't try to install it
+	snapstate.Set(s.state, "snapd", &snapstate.SnapState{
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{
+			{
+				RealName: "snapd",
+				Revision: snap.R(1),
+			},
+		}),
+		Current: snap.R(1),
+	})
+
+	prereqTask := s.state.NewTask("prerequisites", "test")
+	prereqTask.Set("snap-setup", &snapstate.SnapSetup{
+		SideInfo: &snap.SideInfo{
+			RealName: "foo",
+			Revision: snap.R(33),
+		},
+		Base:    "core18",
+		Channel: "stable",
+	})
+
+	linkSnapTask := s.state.NewTask("link-snap", "Doing a fake link-snap")
+	linkSnapTask.Set("snap-setup", &snapstate.SnapSetup{
+		SideInfo: &snap.SideInfo{
+			RealName: "core18",
+			Revision: snap.R(1),
+		},
+		Type:    snap.TypeBase,
+		Channel: "stable",
+	})
+
+	// this simulates the link-snap task being blocked on the prerequisites
+	// task, like during a remodel
+	linkSnapTask.WaitFor(prereqTask)
+
+	// for this test, we don't care about what link-snap does
+	s.runner.AddHandler("link-snap", func(task *state.Task, _ *tomb.Tomb) error {
+		return nil
+	}, nil)
+
+	chg := s.state.NewChange("do-prereqs", "...")
+	chg.AddTask(prereqTask)
+	chg.AddTask(linkSnapTask)
+
+	s.state.Unlock()
+
+	// ensure and wait twice since the link-snap task depends on the
+	// prerequisites task
+	s.se.Ensure()
+	s.se.Wait()
+	s.se.Ensure()
+	s.se.Wait()
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	c.Check(prereqTask.Status(), Equals, state.DoneStatus)
+	c.Check(linkSnapTask.Status(), Equals, state.DoneStatus)
+	c.Check(chg.Err(), IsNil)
+	c.Check(chg.Status(), Equals, state.DoneStatus)
+
+	// core18 should not be installed, despite the prerequisites task being
+	// finished successfully. this is because, during a remodel, we do not wait
+	// for (or attempt) prerequisites installs.
+	var snapst snapstate.SnapState
+	err := snapstate.Get(s.state, "core18", &snapst)
+	c.Check(err, testutil.ErrorIs, state.ErrNoState)
+}
