@@ -75,6 +75,7 @@ func (e *ValidationError) Error() string {
 		for _, err := range e.SystemErrors[s] {
 			fmt.Fprintf(&buf, "\n - %s", err)
 		}
+		first = false
 	}
 	return buf.String()
 }
@@ -96,7 +97,7 @@ func ValidateFromYaml(seedYamlFile string) error {
 	}
 
 	tm := timings.New(nil)
-	if err := seed.LoadMeta(tm); err != nil {
+	if err := seed.LoadMeta(AllModes, nil, tm); err != nil {
 		if missingErr, ok := err.(*essentialSnapMissingError); ok {
 			if seed.Model().Classic() && missingErr.SnapName == "core" {
 				err = fmt.Errorf("essential snap core or snapd must be part of the seed")
@@ -105,13 +106,10 @@ func ValidateFromYaml(seedYamlFile string) error {
 		return newValidationError("", err)
 	}
 
-	// TODO:UC20: make the NumSnaps/Iter part of Seed
-	seed16 := seed.(*seed16)
-
 	ve := &ValidationError{}
 	// read the snap infos
-	snapInfos := make([]*snap.Info, 0, seed16.NumSnaps())
-	seed16.Iter(func(sn *Snap) error {
+	snapInfos := make([]*snap.Info, 0, seed.NumSnaps())
+	seed.Iter(func(sn *Snap) error {
 		snapf, err := snapfile.Open(sn.Path)
 		if err != nil {
 			ve.addErr("", err)
@@ -126,7 +124,8 @@ func ValidateFromYaml(seedYamlFile string) error {
 		return nil
 	})
 
-	if errs2 := snap.ValidateBasesAndProviders(snapInfos); errs2 != nil {
+	// TODO: surface the warnings too, like we do for snap container checks
+	if _, errs2 := snap.ValidateBasesAndProviders(snapInfos); errs2 != nil {
 		ve.addErr("", errs2...)
 	}
 	if ve.hasErrors() {
